/*
 * component.h
 * Copyright 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *     Steve Grubb <sgrubb@redhat.com>
 *     Junji Kanemaru <junji.kanemaru@linuon.com>
 */
#ifndef COMPONENT_H
#define COMPONENT_H
#include <string>
#include <vector>

/**
 * Class Component
 * This class provides the basic interface to the methods that are common
 * to all Components.
 */
class Component {
/**
 * Public stuff
 */
public:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Empty Constructor
	 */
	Component ( ) { }
	virtual ~Component ( ) { }

	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
	/**
	 * This method is called with the configuration string for the component.
	 */
	virtual int init (const char * config) = 0;
		
	
	/**
	 * This method releases all resources that the component is currently using.
	 */
	virtual void destroy () = 0;
		
	
	/**
	 * This method is used to stop the active processing that component
     * may be doing.
	 */
	virtual void suspend () = 0;
		
	
	/**
	 * This method tells the component to start its normal processing of data.
	 */
	virtual void resume () = 0;
		
	
	/**
	 * This method returns the character string that was used to configure
     * the instance.
	 */
	virtual const char * get_config () = 0;

	enum ComponentState {
		IDLE,
		CONFIGURING,
		RUNNING,
		SUSPENDED
	};
	
/**
 * Protected stuff
 */
protected:
	/**
	 * Fields
	 */
	/**
	 * This variable is used to signal to threads what the current
     * status of the component is.
	 */
	 ComponentState _state;

	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of _state
	 * This variable is used to signal to threads what the current
     * status of the component is.
	 * @return the value of _state
	 */
	Component::ComponentState get__state ( );
		
	
	/**
	 * Set the value of _state
	 * This variable is used to signal to threads what the current
     * status of the component is.
	 * @param value the value of _state
	 */
	void set__state (ComponentState value );
		
	
	/**
	 * Operations
	 */
/**
 * Private stuff
 */
private:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
};
#endif //COMPONENT_H

