/*
 * event.h
 * Copyright 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *     Steve Grubb <sgrubb@redhat.com>
 *     Junji Kanemaru <junji.kanemaru@linuon.com>
 */
#ifndef EVENT_H
#define EVENT_H
#include <string>

/* Private headers */
#include "value.h"

/**
 * ******************************* Abstract Class ****************************
 * Event does not have any pure virtual methods, but its author
 *   defined it as an abstract class, so you should not use it directly.
 *   Inherit from it instead and create only objects from the derived classes
 * *****************************************************************************
 */
class Event : public Value {
/**
 * Public stuff
 */
public:
    typedef enum {
        PRIO_LOW,
        PRIO_NORMAL,
        PRIO_HIGH,
        PRIO_CRITICAL
    } Priority;

	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	Event () { prio = PRIO_LOW; }
	Event (const Event& evt);

	Event (const Value& v, Priority prio = PRIO_NORMAL);
	Event (size_t size, const void* ptr, Priority prio = PRIO_NORMAL);
	Event (const std::string& str, Priority prio = PRIO_NORMAL);
	Event (const char* str, Priority prio = PRIO_NORMAL);

	/**
	 * Accessor Methods
	 */
    Priority getPriority() const { return prio; }

	/**
	 * Operations
	 */
/**
 * Protected stuff
 */
protected:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
/**
 * Private stuff
 */
private:
	/**
	 * Fields
	 */
    Priority prio;
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
};
#endif //EVENT_H

