#include <stdio.h>
#include <unistd.h>
#include <iostream>
#include <string>
#include <vector>
#include <typeinfo>
#include <cxxabi.h>

#include "fault.h"
#include "value.h"
#include "event.h"
#include "circularqueue.h"

using namespace std;

void regular_type_test() {
    cout << ">>>>>>> Regular type test start" << endl;

    Value v;
    cout << "v: " << v.getTypeString() << endl;

    ValueBoolean vb(-1);
    cout << "vb: " << typeid(vb).name() << endl;

    cout << "bool: " << (bool)vb << endl;
    vb = 100;
    cout << "bool: " << (bool)vb << endl;
    vb = 0;
    cout << "bool: " << (bool)vb << endl;

    string s;

    ValueString vs1("value string 1");
    s = vs1.getString();
    cout << "length of vs1 = " << vs1.size() << endl;
    ValueString vs2("value string 2");
    ValueInt32 vi(100);
    v = vs2;
    cout << vs1.getString() << endl;
    cout << (const char*)vs2 << endl;
    cout << (string)vs2 << endl;
    cout << (int)vi << endl;

    cout << "v: " << v.getTypeString() << endl;
    v = Value::VOID_VALUE;
    cout << "v: " << v.getTypeString() << endl;
    v = vs2;

    Value vv(vs1);
    cout << "vv: " << vv.getString() << endl;

    cout << s << endl;
    cout << (const char*)v << endl;

    cout << "<<<<<<< Regular type test done" << endl << endl;
}

void array_test() {
    cout << ">>>>>>> Array type test start" << endl;
    Value v;
{
    ValueArray va;

    ValueString vs1("value string 1");
    ValueString vs2("value string 2");
    ValueInt32 vi(100);

    v = vs1;
    cout << "v <- vs1: " << (const char*)v << endl;
    v = vs2;
    cout << "v <- vs2: " << (const char*)v << endl;
    v = vi;
    cout << "v <- vi: " << (int)v << endl;

    va.append(vs1); // item #0
    va.append(vs2); // item #1
    va.append(vi); // item #2

    va += vs2; // itehm #3
    va += ValueString("blabla"); // itehm #4

    Value vi2 = vi;
    va.append(vi2); // item #5
    va -= vi2; // <- this removes item #2 since vi2 == vi and item #5 will become #4

    cout << "size of array = " << va.size() << endl;
    cout << (const char*)va[0] << endl;
    cout << (const char*)va[1] << endl;
    cout << (const char*)va[2] << endl;
    cout << (const char*)va[3] << endl;
    cout << (int)va[4] << endl;

    v = va;
    va += vi2;
}
    if (v == Value::ARRAY) {
        cout << "v is ARRAY" << endl;
        cout << "size of array = " << v.size() << endl;
        cout << (const char*)v[0] << endl;
        cout << (const char*)v[1] << endl;
        cout << (const char*)v[2] << endl;
        cout << (const char*)v[3] << endl;
        cout << (int)v[4] << endl;
    }

    cout << "<<<<<<< Array type test done" << endl << endl;
}

void structure_test() {
    cout << ">>>>>>> Structure type test start" << endl;
    ValueStruct vs;
{
    ValueArray va;
    Value v;

    ValueString vs1("value string 1");
    ValueString vs2("value string 2");
    ValueInt32 vi(100);

    vs.set("key1", vs1);
    vs.set("key2", vs2);
    vs.set("key3", vi);
    vs["key1"] = vs2;

    va.append(vs1);
    va.append(vs2);
    va.append(vi);
    vs.set("key4", va);

    cout << (const char*)vs["key1"] << endl;
    cout << (const char*)vs["key2"] << endl;
    cout << (int)vs["key3"] << endl;

    cout << (const char*)vs["key4"][0] << endl;
    cout << (const char*)vs["key4"][1] << endl;
    cout << (int)vs["key4"][2] << endl;

    v = vs;

    vs.erase("key1");

    if (v == Value::STRUCT) {
        cout << "v is Value::STRUCT" << endl;
        //cout << (const char*)v["key1"] << endl;
        cout << (const char*)v["key2"] << endl;
        cout << (int)v["key3"] << endl;

        Value* vp = new Value(v);
        cout << "vp-> " << (const char*)(*vp)["key4"][0] << endl;
        cout << "vp-> " << (const char*)(*vp)["key4"][1] << endl;
        cout << "vp-> " << (int)(*vp)["key4"][2] << endl;
        delete vp;
    }   
}
    cout << "All other Value objects are deleted at this point except \"vs\""
        << endl;
    ValueString ss = vs["key2"];
    cout << "ss: " << (const char*)ss << endl;

    cout << "<<<<<<< Structure type test done" << endl << endl;
}

void queue_test() {
    cout << ">>>>>>> CircularPriorityQueue test start" << endl;

    CircularPriorityQueue Q(3, 3, 3, 3);

    Event evt;
    Value v;

    v = ValueString("First Event(Event::PRIO_NORMAL)");
    Q.enqueue(Event(v));

    v = ValueString("Second Event(Event::PRIO_HIGH)");
    Q.enqueue(Event(v, Event::PRIO_HIGH));

    v = ValueInt32(3);
    Q.enqueue(Event(v, Event::PRIO_LOW));

    ValueArray va;
    va.append(ValueString("First array item"));
    va.append(ValueString("Second array item"));
    va.append(ValueInt32(4));
    Q.enqueue(Event(va, Event::PRIO_NORMAL));

    ValueStruct vs;
    vs.set("key1", ValueString("First struct item"));
    vs.set("key2", ValueString("Second struct item"));
    vs.set("key3", ValueString("Third struct item"));
    vs.set("key4", ValueInt32(5));
    Q.enqueue(Event(vs, Event::PRIO_CRITICAL));

    Event evt2 = Event(ValueString("evt2: Event::PRIO_CRITICAL"), 
            Event::PRIO_CRITICAL);
    Q.enqueue(evt2);
    evt2 = evt;
    Q.enqueue(evt2);

    size_t size = Q.count();
    cout << "Q size = " << size << endl;

    while (size--) {
        if (Q.dequeue(evt)) {
            switch(evt.type()) {
                case Value::STRING:
                    cout << "v: " << (string)evt << endl;
                    break;
                case Value::INT32:
                    cout << "v: " << (int)evt << endl;
                    break;
                case Value::ARRAY: {
                    size_t size = evt.size();
                    for (size_t i=0; i<size; i++) {
                        if (evt[i].type() == Value::STRING) {
                            cout << "v: " << (string)evt[i] << endl;
                        }
                        else {
                            cout << "v: " << (int)evt[i] << endl;
                        }
                    }
                    break;
                }
                case Value::STRUCT: {
                    cout << "v: " << "Value::STRUCT"  << endl;
                    break;
                }
                default:
                    cout << "v: " << "Don't care: type " << evt.type() << endl;
                    break;
            }
        }
        else {
            cout << "Queue empty" << endl;
        }
    }

    cout << "<<<<<<< CircularPriorityQueue test done" << endl << endl;
}

int main(int ac, char** av) {
    size_t itertaions = av[1] ? atol(av[1]) : 1;
    do {
        try {
            regular_type_test();
            array_test();
            structure_test();
            queue_test();
        }
        catch (Fault& fault) {
            cout << fault.file() << ":" << fault.line() << ": " << fault.message() << endl;
            break;
        }
        usleep(500);
    } while (--itertaions);
}

