/*
 * thread.h
 * Copyright 2005 Linuon Inc. Tokyo Japan All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *     Junji Kanemaru <junji.kanemaru@linuon.com>
 */
#ifndef THREAD_H
#define THREAD_H
#include <pthread.h>

class Thread {
public:
    typedef enum {
        PRIO_LOW,
        PRIO_NORMAL,
        PRIO_HIGH
    } Priority;

    typedef enum {
        TH_READY,
        TH_RUNNING,
        TH_SUSPENDED,
        TH_DONE,
        TH_CANCELED
    } Status;

    Thread();
    virtual ~Thread();

    void start(void* userData, Priority prio = PRIO_NORMAL);
    void cancel();
    void suspend();
    void resume();
    bool finished() { return (stat == TH_DONE || stat == TH_CANCELED); }
    int wait(time_t second = 0); /* second = 0 means forever */
    int status() { return stat; }
    int exitCode() { return ret; }

protected:
    /**
    ** mainHandler is pure virtual method. Must be defined in derived class.
    ** It takes int& stat and void* userData. Returns bool, true to continue
    ** thread process, false to exit thread. The return code status can be
    ** read by calling exitCode() method. */
    virtual bool mainHandler(int& stat, void* userData) = 0;
    void* userData;

private:
    static void* threadEntry(void* _this);
    static void threadCancelHandler(void* _this);

    pthread_t thID;
    pthread_mutex_t mutex;
    pthread_cond_t cond;

    Status stat, statSaved;
    time_t sleepTime;
    Priority prio;
    int ret;
};

#endif /* THREAD_H */

